let defaultValue = {
  hh: "00",
  mm: "00",
  ss: "00"
};
const hhInMilli = 3600000,
      mmInMilli = 60000,
      ssInMilli = 1000;

function Timer() {
  this.timerDetails = {};

  this.notifier = function () {};

  this.clientData = {};
}

Timer.timeObjToMillis = function (duration = {}) {
  let {
    hh = "00",
    mm = "00",
    ss = "00"
  } = duration;
  return Number(hh) * hhInMilli + Number(mm) * mmInMilli + Number(ss) * ssInMilli;
};

Timer.millisToTimeObj = function (millis) {
  let hh = `${Math.floor(millis / hhInMilli)}`,
      mm = `${Math.floor(millis % hhInMilli / mmInMilli)}`,
      ss = `${Math.floor(millis % hhInMilli % mmInMilli / ssInMilli)}`;
  hh = hh.length === 1 ? `0${hh}` : hh;
  mm = mm.length === 1 ? `0${mm}` : mm;
  ss = ss.length === 1 ? `0${ss}` : ss;
  return {
    hh,
    mm,
    ss
  };
};

Timer.prototype = {
  init: function (details) {
    let {
      data,
      notifier
    } = details;
    this.initTimerDetails(data);
    this.initClientData(data);
    this.setNotifier(notifier);
    Object.keys(data).map(detailKey => {
      let {
        time,
        status
      } = data[detailKey];
      this.updateTimerState(detailKey, undefined, time, status);
      this.deleteTimerDetailKey(detailKey, "isInitCall");
    });
  },
  initClientData: function (data) {
    Object.keys(data).map(detailKey => {
      this.clientData[detailKey] = {};
    });
  },
  setNotifier: function (func) {
    this.notifier = func;
  },
  clearInterval: function (key) {
    let timerDetail = this.getTimerDetail(key);
    clearInterval(timerDetail.intervalId);
    delete timerDetail.intervalId;
  },
  initTimerDetails: function (details) {
    Object.keys(details).map(detailKey => {
      this.timerDetails[detailKey] = {
        isInitCall: true
      };
    });
  },
  getTimerDetail: function (key) {
    return this.timerDetails[key];
  },
  notify: function (key) {
    this.notifier(key, this.clientData[key]);
  },
  setTimerDetail: function (key, obj) {
    this.timerDetails[key] = Object.assign({}, this.timerDetails[key], obj);
  },
  deleteTimerDetailKey: function (key, detailKey) {
    let newObj = Object.assign({}, this.timerDetails[key]);
    delete newObj[detailKey];
    this.timerDetails[key] = newObj;
  },
  setClientData: function (key, obj) {
    let newData = {};
    Object.keys(this.clientData).map(clientKey => {
      newData[clientKey] = clientKey !== key ? this.clientData[clientKey] : Object.assign({}, this.clientData[key], obj);
    });
    this.clientData = newData;
  },
  updateTimerState: function (key, action, duration = defaultValue, status) {
    if (action === "START") {
      this.setTimerDetail(key, {
        previousTimeObj: duration
      });
      this.start(key);
    } else if (action === "PAUSE" || status === 'PAUSED') {
      this.setTimerDetail(key, {
        previousTimeObj: duration
      });
      this.pause(key);
    } else if (action === "RESUME" || status === 'RUNNING') {
      this.setTimerDetail(key, {
        previousTimeObj: duration
      });
      this.resume(key);
    } else if (action === "INIT" || action === "STOP" || action === "RESET" || status == 'INIT') {
      this.reset(key);
    }
  },
  triggerTimerMechanism: function (key) {
    let intervalId = setInterval(() => {
      let {
        startTime
      } = this.getTimerDetail(key);
      let time = Timer.millisToTimeObj(new Date().getTime() - startTime);
      this.setClientData(key, {
        time
      });
      this.notify(key);
    }, 1000);
    this.setTimerDetail(key, {
      intervalId
    });
  },
  start: function (key) {
    this.clearInterval(key);
    let {
      previousTimeObj,
      isInitCall
    } = this.getTimerDetail(key);
    let startTime = new Date().getTime() - Timer.timeObjToMillis(previousTimeObj);
    this.deleteTimerDetailKey(key, "previousTimeObj");
    this.setTimerDetail(key, {
      startTime
    });
    this.setClientData(key, {
      status: "RUNNING",
      time: Timer.millisToTimeObj(new Date().getTime() - startTime)
    });
    this.triggerTimerMechanism(key);
    isInitCall != true && this.notify(key);
  },
  pause: function (key) {
    this.clearInterval(key);
    let currentTime = new Date().getTime();
    let {
      startTime = currentTime,
      isInitCall,
      previousTimeObj
    } = this.getTimerDetail(key);
    let previousTime = Timer.millisToTimeObj(previousTimeObj ? Timer.timeObjToMillis(previousTimeObj) : currentTime - startTime);
    this.setTimerDetail(key, {
      previousTimeObj: previousTime
    });
    this.setClientData(key, {
      status: "PAUSED",
      time: previousTime
    });
    isInitCall != true && this.notify(key);
  },
  resume: function (key) {
    this.clearInterval(key);
    let {
      previousTimeObj,
      isInitCall
    } = this.getTimerDetail(key),
        currentTime = new Date().getTime();
    let startTime = currentTime - Timer.timeObjToMillis(previousTimeObj);
    this.setTimerDetail(key, {
      startTime
    });
    this.setClientData(key, {
      status: "RUNNING",
      time: previousTimeObj
    });
    this.deleteTimerDetailKey(key, "previousTimeObj");
    this.triggerTimerMechanism(key);
    isInitCall != true && this.notify(key);
  },
  reset: function (key) {
    this.clearInterval(key);
    this.setClientData(key, {
      status: "INIT",
      time: {
        hh: "00",
        mm: "00",
        ss: "00"
      }
    });
    this.deleteTimerDetailKey(key, "previousTimeObj");
    this.deleteTimerDetailKey(key, "startTime");
    this.getTimerDetail(key).isInitCall != true && this.notify(key);
  }
};
export default Timer;